import numpy as np            # pylint: disable=unused-import
import typing                 # pylint: disable=unused-import
from nomad.metainfo import (  # pylint: disable=unused-import
    MSection, MCategory, Category, Package, Quantity, Section, SubSection, SectionProxy,
    Reference
)
from nomad.metainfo.legacy import LegacyDefinition

from nomad.datamodel.metainfo import public

m_package = Package(
    name='siesta_autogenerated_nomadmetainfo_json',
    description='None',
    a_legacy=LegacyDefinition(name='siesta.autogenerated.nomadmetainfo.json'))


class x_siesta_input(MCategory):
    '''
    siesta input variables
    '''

    m_def = Category(
        a_legacy=LegacyDefinition(name='x_siesta_input'))


class x_siesta_section_input(MSection):
    '''
    input section
    '''

    m_def = Section(validate=False, a_legacy=LegacyDefinition(name='x_siesta_section_input'))

    x_siesta_input_endblock = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "%endblock"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_endblock'))

    x_siesta_input_Atom_Setup_Only = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Atom-Setup-Only"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Atom-Setup-Only'))

    x_siesta_input_Atom_Debug_KB_Generation = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Atom.Debug.KB.Generation"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Atom.Debug.KB.Generation'))

    x_siesta_input_AtomCoorFormatOut = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "AtomCoorFormatOut"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_AtomCoorFormatOut'))

    x_siesta_input_AtomLeftVcte = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "AtomLeftVcte"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_AtomLeftVcte'))

    x_siesta_input_AtomRightVcte = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "AtomRightVcte"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_AtomRightVcte'))

    x_siesta_input_AtomicCoordinatesFormat = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "AtomicCoordinatesFormat"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_AtomicCoordinatesFormat'))

    x_siesta_input_BasisPressure = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BasisPressure"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BasisPressure'))

    x_siesta_input_BornCharge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BornCharge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BornCharge'))

    x_siesta_input_BuildSuperCell = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BuildSuperCell"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BuildSuperCell'))

    x_siesta_input_BulkLead = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BulkLead"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BulkLead'))

    x_siesta_input_BulkTransport = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BulkTransport"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BulkTransport'))

    x_siesta_input_BulkTransvCellSize = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BulkTransvCellSize"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BulkTransvCellSize'))

    x_siesta_input_BulkTransvCellSizeX = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BulkTransvCellSizeX"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BulkTransvCellSizeX'))

    x_siesta_input_BulkTransvCellSizeY = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BulkTransvCellSizeY"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BulkTransvCellSizeY'))

    x_siesta_input_BulkTransvCellSizeZ = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "BulkTransvCellSizeZ"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_BulkTransvCellSizeZ'))

    x_siesta_input_CB_MaxKappa = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "CB.MaxKappa"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_CB.MaxKappa'))

    x_siesta_input_CB_WriteComplexBands = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "CB.WriteComplexBands"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_CB.WriteComplexBands'))

    x_siesta_input_CDFT = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "CDFT"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_CDFT'))

    x_siesta_input_CDFT_MaxIter = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "CDFT.MaxIter"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_CDFT.MaxIter'))

    x_siesta_input_COOP_Write = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "COOP.Write"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_COOP.Write'))

    x_siesta_input_CalcIETS = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "CalcIETS"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_CalcIETS'))

    x_siesta_input_CalcMPSH = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "CalcMPSH"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_CalcMPSH'))

    x_siesta_input_ChangeKgridInMD = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ChangeKgridInMD"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ChangeKgridInMD'))

    x_siesta_input_Compat_pre_v4_DM_H = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Compat-pre-v4-DM-H"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Compat-pre-v4-DM-H'))

    x_siesta_input_DM_AllowExtrapolation = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.AllowExtrapolation"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.AllowExtrapolation'))

    x_siesta_input_DM_AllowReuse = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.AllowReuse"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.AllowReuse'))

    x_siesta_input_DM_EnergyTolerance = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.EnergyTolerance"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.EnergyTolerance'))

    x_siesta_input_DM_FIRE_Mixing = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.FIRE.Mixing"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.FIRE.Mixing'))

    x_siesta_input_DM_FormattedFiles = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.FormattedFiles"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.FormattedFiles'))

    x_siesta_input_DM_FormattedInput = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.FormattedInput"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.FormattedInput'))

    x_siesta_input_DM_FormattedOutput = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.FormattedOutput"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.FormattedOutput'))

    x_siesta_input_DM_HarrisTolerance = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.HarrisTolerance"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.HarrisTolerance'))

    x_siesta_input_DM_KickMixingWeight = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.KickMixingWeight"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.KickMixingWeight'))

    x_siesta_input_DM_MixSCF1 = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.MixSCF1"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.MixSCF1'))

    x_siesta_input_DM_MixingWeight = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.MixingWeight"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.MixingWeight'))

    x_siesta_input_DM_NormalizationTolerance = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.NormalizationTolerance"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.NormalizationTolerance'))

    x_siesta_input_DM_NormalizeDuringSCF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.NormalizeDuringSCF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.NormalizeDuringSCF'))

    x_siesta_input_DM_NumberBroyden = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.NumberBroyden"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.NumberBroyden'))

    x_siesta_input_DM_NumberKick = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.NumberKick"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.NumberKick'))

    x_siesta_input_DM_NumberPulay = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.NumberPulay"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.NumberPulay'))

    x_siesta_input_DM_OccupancyTolerance = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.OccupancyTolerance"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.OccupancyTolerance'))

    x_siesta_input_DM_Pulay_Avoid_First_After_Kick = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.Pulay.Avoid.First.After.Kick"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.Pulay.Avoid.First.After.Kick'))

    x_siesta_input_DM_PulayOnFile = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.PulayOnFile"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.PulayOnFile'))

    x_siesta_input_DM_RequireEnergyConvergence = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.RequireEnergyConvergence"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.RequireEnergyConvergence'))

    x_siesta_input_DM_RequireHarrisConvergence = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.RequireHarrisConvergence"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.RequireHarrisConvergence'))

    x_siesta_input_DM_Tolerance = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.Tolerance"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.Tolerance'))

    x_siesta_input_DM_UseSaveDM = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DM.UseSaveDM"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DM.UseSaveDM'))

    x_siesta_input_Delta = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Delta"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Delta'))

    x_siesta_input_DeltaWorkfunction = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DeltaWorkfunction"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DeltaWorkfunction'))

    x_siesta_input_Diag_AllInOne = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Diag.AllInOne"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Diag.AllInOne'))

    x_siesta_input_Diag_DivideAndConquer = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Diag.DivideAndConquer"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Diag.DivideAndConquer'))

    x_siesta_input_Diag_Memory = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Diag.Memory"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Diag.Memory'))

    x_siesta_input_Diag_NoExpert = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Diag.NoExpert"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Diag.NoExpert'))

    x_siesta_input_Diag_ParallelOverK = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Diag.ParallelOverK"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Diag.ParallelOverK'))

    x_siesta_input_Diag_PreRotate = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Diag.PreRotate"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Diag.PreRotate'))

    x_siesta_input_Diag_Use2D = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Diag.Use2D"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Diag.Use2D'))

    x_siesta_input_DiagMemory = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DiagMemory"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DiagMemory'))

    x_siesta_input_DiagScale = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DiagScale"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DiagScale'))

    x_siesta_input_DirectPhi = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DirectPhi"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DirectPhi'))

    x_siesta_input_DivideAndConquer = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "DivideAndConquer"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_DivideAndConquer'))

    x_siesta_input_EM_AddRhoGate = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.AddRhoGate"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.AddRhoGate'))

    x_siesta_input_EM_AddVgIsolatedLocalCharges = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.AddVgIsolatedLocalCharges"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.AddVgIsolatedLocalCharges'))

    x_siesta_input_EM_COOPCalculate = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.COOPCalculate"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.COOPCalculate'))

    x_siesta_input_EM_COOPNumberOfBonds = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.COOPNumberOfBonds"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.COOPNumberOfBonds'))

    x_siesta_input_EM_DebugRhoGate = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.DebugRhoGate"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.DebugRhoGate'))

    x_siesta_input_EM_NetRhoGateCharge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.NetRhoGateCharge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.NetRhoGateCharge'))

    x_siesta_input_EM_PrintLimits = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.PrintLimits"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.PrintLimits'))

    x_siesta_input_EM_RhoGateLxMax = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.RhoGateLxMax"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.RhoGateLxMax'))

    x_siesta_input_EM_RhoGateLxMin = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.RhoGateLxMin"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.RhoGateLxMin'))

    x_siesta_input_EM_RhoGateLyMax = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.RhoGateLyMax"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.RhoGateLyMax'))

    x_siesta_input_EM_RhoGateLyMin = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.RhoGateLyMin"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.RhoGateLyMin'))

    x_siesta_input_EM_RhoGateLzMax = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.RhoGateLzMax"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.RhoGateLzMax'))

    x_siesta_input_EM_RhoGateLzMin = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.RhoGateLzMin"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.RhoGateLzMin'))

    x_siesta_input_EM_TRCAddVCDFT = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.TRCAddVCDFT"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.TRCAddVCDFT'))

    x_siesta_input_EM_TimeReversal = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.TimeReversal"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.TimeReversal'))

    x_siesta_input_EM_Timings = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.Timings"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.Timings'))

    x_siesta_input_EM_addV = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EM.addV"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EM.addV'))

    x_siesta_input_EMPDOSKSO = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EMPDOSKSO"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EMPDOSKSO'))

    x_siesta_input_EMTransport = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EMTransport"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EMTransport'))

    x_siesta_input_ElectronicTemperature = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ElectronicTemperature"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ElectronicTemperature'))

    x_siesta_input_EnergLowestBound = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "EnergLowestBound"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_EnergLowestBound'))

    x_siesta_input_FilterCutoff = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "FilterCutoff"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_FilterCutoff'))

    x_siesta_input_FilterTol = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "FilterTol"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_FilterTol'))

    x_siesta_input_FinalTransmRange = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "FinalTransmRange"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_FinalTransmRange'))

    x_siesta_input_FixAuxiliaryCell = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "FixAuxiliaryCell"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_FixAuxiliaryCell'))

    x_siesta_input_FixAuxillaryCell = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "FixAuxillaryCell"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_FixAuxillaryCell'))

    x_siesta_input_FixSpin = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "FixSpin"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_FixSpin'))

    x_siesta_input_ForceAuxCell = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ForceAuxCell"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ForceAuxCell'))

    x_siesta_input_FullRamp = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "FullRamp"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_FullRamp'))

    x_siesta_input_HSetupOnly = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "HSetupOnly"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_HSetupOnly'))

    x_siesta_input_Harris_functional = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Harris_functional"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Harris_functional'))

    x_siesta_input_HartreeLeadsBottom = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "HartreeLeadsBottom"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_HartreeLeadsBottom'))

    x_siesta_input_HartreeLeadsLeft = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "HartreeLeadsLeft"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_HartreeLeadsLeft'))

    x_siesta_input_HartreeLeadsRight = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "HartreeLeadsRight"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_HartreeLeadsRight'))

    x_siesta_input_Ik_Select = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Ik_Select"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Ik_Select'))

    x_siesta_input_InitTransmRange = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "InitTransmRange"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_InitTransmRange'))

    x_siesta_input_InitTransport = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "InitTransport"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_InitTransport'))

    x_siesta_input_KB_New_Reference_Orbitals = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "KB.New.Reference.Orbitals"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_KB.New.Reference.Orbitals'))

    x_siesta_input_KB_Rmax = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "KB.Rmax"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_KB.Rmax'))

    x_siesta_input_LDAU_units = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "LDAU.units"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_LDAU.units'))

    x_siesta_input_LDAUForces = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "LDAUForces"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_LDAUForces'))

    x_siesta_input_LDAU_METHOD = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "LDAU_METHOD"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_LDAU_METHOD'))

    x_siesta_input_LatticeConstant = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "LatticeConstant"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_LatticeConstant'))

    x_siesta_input_LongOutput = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "LongOutput"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_LongOutput'))

    x_siesta_input_MD_AnnealOption = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.AnnealOption"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.AnnealOption'))

    x_siesta_input_MD_BulkModulus = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.BulkModulus"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.BulkModulus'))

    x_siesta_input_MD_FCAcousticSumRule = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FCAcousticSumRule"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FCAcousticSumRule'))

    x_siesta_input_MD_FCAtomRestart = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FCAtomRestart"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FCAtomRestart'))

    x_siesta_input_MD_FCAxisRestart = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FCAxisRestart"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FCAxisRestart'))

    x_siesta_input_MD_FCDispl = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FCDispl"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FCDispl'))

    x_siesta_input_MD_FCEigenVectors = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FCEigenVectors"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FCEigenVectors'))

    x_siesta_input_MD_FCIR = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FCIR"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FCIR'))

    x_siesta_input_MD_FCRead = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FCRead"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FCRead'))

    x_siesta_input_MD_FCfirst = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FCfirst"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FCfirst'))

    x_siesta_input_MD_FClast = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FClast"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FClast'))

    x_siesta_input_MD_FinalTimeStep = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FinalTimeStep"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FinalTimeStep'))

    x_siesta_input_MD_FireQuench = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.FireQuench"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.FireQuench'))

    x_siesta_input_MD_InitialTemperature = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.InitialTemperature"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.InitialTemperature'))

    x_siesta_input_MD_InitialTimeStep = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.InitialTimeStep"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.InitialTimeStep'))

    x_siesta_input_MD_LengthTimeStep = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.LengthTimeStep"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.LengthTimeStep'))

    x_siesta_input_MD_MaxCGDispl = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.MaxCGDispl"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.MaxCGDispl'))

    x_siesta_input_MD_MaxForceTol = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.MaxForceTol"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.MaxForceTol'))

    x_siesta_input_MD_MaxStressTol = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.MaxStressTol"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.MaxStressTol'))

    x_siesta_input_MD_NoseMass = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.NoseMass"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.NoseMass'))

    x_siesta_input_MD_NumCGsteps = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.NumCGsteps"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.NumCGsteps'))

    x_siesta_input_MD_NumNEBsteps = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.NumNEBsteps"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.NumNEBsteps'))

    x_siesta_input_MD_ParrinelloRahmanMass = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.ParrinelloRahmanMass"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.ParrinelloRahmanMass'))

    x_siesta_input_MD_Quench = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.Quench"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.Quench'))

    x_siesta_input_MD_RelaxCellOnly = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.RelaxCellOnly"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.RelaxCellOnly'))

    x_siesta_input_MD_RemoveIntraMolecularPressure = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.RemoveIntraMolecularPressure"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.RemoveIntraMolecularPressure'))

    x_siesta_input_MD_TRCSampling = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.TRCSampling"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.TRCSampling'))

    x_siesta_input_MD_TRCSkip = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.TRCSkip"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.TRCSkip'))

    x_siesta_input_MD_TargetPressure = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.TargetPressure"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.TargetPressure'))

    x_siesta_input_MD_TargetTemperature = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.TargetTemperature"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.TargetTemperature'))

    x_siesta_input_MD_TauRelax = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.TauRelax"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.TauRelax'))

    x_siesta_input_MD_Timing = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.Timing"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.Timing'))

    x_siesta_input_MD_TypeOfRun = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.TypeOfRun"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.TypeOfRun'))

    x_siesta_input_MD_UseSaveCG = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.UseSaveCG"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.UseSaveCG'))

    x_siesta_input_MD_UseSaveNEB = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.UseSaveNEB"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.UseSaveNEB'))

    x_siesta_input_MD_UseSaveXV = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.UseSaveXV"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.UseSaveXV'))

    x_siesta_input_MD_UseSaveZM = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.UseSaveZM"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.UseSaveZM'))

    x_siesta_input_MD_UseStructFile = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.UseStructFile"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.UseStructFile'))

    x_siesta_input_MD_VariableCell = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MD.VariableCell"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MD.VariableCell'))

    x_siesta_input_MM_Cutoff = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MM.Cutoff"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MM.Cutoff'))

    x_siesta_input_MM_UnitsDistance = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MM.UnitsDistance"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MM.UnitsDistance'))

    x_siesta_input_MM_UnitsEnergy = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MM.UnitsEnergy"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MM.UnitsEnergy'))

    x_siesta_input_MPSHAtomFirst = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MPSHAtomFirst"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MPSHAtomFirst'))

    x_siesta_input_MPSHAtomLast = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MPSHAtomLast"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MPSHAtomLast'))

    x_siesta_input_MPSHOrbFirst = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MPSHOrbFirst"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MPSHOrbFirst'))

    x_siesta_input_MPSHOrbLast = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MPSHOrbLast"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MPSHOrbLast'))

    x_siesta_input_MaxBondDistance = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MaxBondDistance"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MaxBondDistance'))

    x_siesta_input_MaxSCFIterations = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MaxSCFIterations"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MaxSCFIterations'))

    x_siesta_input_MeshCutoff = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MeshCutoff"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MeshCutoff'))

    x_siesta_input_MeshSubDivisions = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MeshSubDivisions"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MeshSubDivisions'))

    x_siesta_input_MinSCFIterations = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MinSCFIterations"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MinSCFIterations'))

    x_siesta_input_MixCharge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MixCharge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MixCharge'))

    x_siesta_input_MixHamiltonian = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MixHamiltonian"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MixHamiltonian'))

    x_siesta_input_MixedParallel = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MixedParallel"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MixedParallel'))

    x_siesta_input_MonitorForcesInSCF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MonitorForcesInSCF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MonitorForcesInSCF'))

    x_siesta_input_MullikenInSCF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "MullikenInSCF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_MullikenInSCF'))

    x_siesta_input_NC_OrbitalRotationEnd = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NC.OrbitalRotationEnd"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NC.OrbitalRotationEnd'))

    x_siesta_input_NC_OrbitalRotationStart = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NC.OrbitalRotationStart"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NC.OrbitalRotationStart'))

    x_siesta_input_NEB_SkipEdge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NEB.SkipEdge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NEB.SkipEdge'))

    x_siesta_input_NEnergReal = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NEnergReal"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NEnergReal'))

    x_siesta_input_NIVPoints = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NIVPoints"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NIVPoints'))

    x_siesta_input_NPoles = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NPoles"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NPoles'))

    x_siesta_input_NSlices = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NSlices"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NSlices'))

    x_siesta_input_NTransmPoints = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NTransmPoints"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NTransmPoints'))

    x_siesta_input_NaiveAuxiliaryCell = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NaiveAuxiliaryCell"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NaiveAuxiliaryCell'))

    x_siesta_input_NeglNonOverlapInt = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NeglNonOverlapInt"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NeglNonOverlapInt'))

    x_siesta_input_NenergImCircle = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NenergImCircle"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NenergImCircle'))

    x_siesta_input_NenergImLine = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NenergImLine"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NenergImLine'))

    x_siesta_input_NetCharge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NetCharge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NetCharge'))

    x_siesta_input_NonCollinearSpin = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NonCollinearSpin"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NonCollinearSpin'))

    x_siesta_input_NumSkipWriteDM = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NumSkipWriteDM"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NumSkipWriteDM'))

    x_siesta_input_NumberLinearMix = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NumberLinearMix"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NumberLinearMix'))

    x_siesta_input_NumberOfAtoms = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NumberOfAtoms"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NumberOfAtoms'))

    x_siesta_input_NumberOfEigenStates = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NumberOfEigenStates"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NumberOfEigenStates'))

    x_siesta_input_NumberOfSpecies = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "NumberOfSpecies"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_NumberOfSpecies'))

    x_siesta_input_Number_of_species = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Number_of_species"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Number_of_species'))

    x_siesta_input_ON_ChemicalPotential = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.ChemicalPotential"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.ChemicalPotential'))

    x_siesta_input_ON_ChemicalPotentialOrder = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.ChemicalPotentialOrder"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.ChemicalPotentialOrder'))

    x_siesta_input_ON_ChemicalPotentialRc = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.ChemicalPotentialRc"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.ChemicalPotentialRc'))

    x_siesta_input_ON_ChemicalPotentialTemperature = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.ChemicalPotentialTemperature"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.ChemicalPotentialTemperature'))

    x_siesta_input_ON_ChemicalPotentialUse = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.ChemicalPotentialUse"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.ChemicalPotentialUse'))

    x_siesta_input_ON_MaxNumIter = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.MaxNumIter"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.MaxNumIter'))

    x_siesta_input_ON_UseSaveLWF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.UseSaveLWF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.UseSaveLWF'))

    x_siesta_input_ON_eta = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.eta"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.eta'))

    x_siesta_input_ON_eta_alpha = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.eta_alpha"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.eta_alpha'))

    x_siesta_input_ON_eta_beta = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.eta_beta"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.eta_beta'))

    x_siesta_input_ON_etol = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.etol"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.etol'))

    x_siesta_input_ON_functional = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ON.functional"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ON.functional'))

    x_siesta_input_OccupationFunction = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "OccupationFunction"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_OccupationFunction'))

    x_siesta_input_On_RcLWF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "On.RcLWF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_On.RcLWF'))

    x_siesta_input_OpticalCalculation = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "OpticalCalculation"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_OpticalCalculation'))

    x_siesta_input_Optim_Broyden = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Optim.Broyden"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Optim.Broyden'))

    x_siesta_input_Output_Structure_Only = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Output-Structure-Only"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Output-Structure-Only'))

    x_siesta_input_PAO_BasisSize = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.BasisSize"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.BasisSize'))

    x_siesta_input_PAO_BasisType = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.BasisType"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.BasisType'))

    x_siesta_input_PAO_EnergyShift = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.EnergyShift"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.EnergyShift'))

    x_siesta_input_PAO_Filter = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.Filter"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.Filter'))

    x_siesta_input_PAO_FixSplitTable = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.FixSplitTable"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.FixSplitTable'))

    x_siesta_input_PAO_Keep_Findp_Bug = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.Keep.Findp.Bug"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.Keep.Findp.Bug'))

    x_siesta_input_PAO_NewSplitCode = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.NewSplitCode"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.NewSplitCode'))

    x_siesta_input_PAO_OldStylePolorbs = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.OldStylePolorbs"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.OldStylePolorbs'))

    x_siesta_input_PAO_SoftDefault = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.SoftDefault"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.SoftDefault'))

    x_siesta_input_PAO_SoftInnerRadius = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.SoftInnerRadius"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.SoftInnerRadius'))

    x_siesta_input_PAO_SoftPotential = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.SoftPotential"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.SoftPotential'))

    x_siesta_input_PAO_SplitNorm = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.SplitNorm"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.SplitNorm'))

    x_siesta_input_PAO_SplitNormH = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.SplitNormH"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.SplitNormH'))

    x_siesta_input_PAO_SplitTailNorm = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PAO.SplitTailNorm"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PAO.SplitTailNorm'))

    x_siesta_input_PS_SIC = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PS.SIC"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PS.SIC'))

    x_siesta_input_ParallelOverK = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ParallelOverK"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ParallelOverK'))

    x_siesta_input_PartialChargesAtEveryGeometry = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PartialChargesAtEveryGeometry"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PartialChargesAtEveryGeometry'))

    x_siesta_input_PartialChargesAtEveryScfStep = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PartialChargesAtEveryScfStep"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PartialChargesAtEveryScfStep'))

    x_siesta_input_PoissonMultigrid = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "PoissonMultigrid"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_PoissonMultigrid'))

    x_siesta_input_Print_ldau = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Print_ldau"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Print_ldau'))

    x_siesta_input_ProcessorGridX = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ProcessorGridX"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ProcessorGridX'))

    x_siesta_input_ProcessorGridY = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ProcessorGridY"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ProcessorGridY'))

    x_siesta_input_ProcessorGridZ = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ProcessorGridZ"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ProcessorGridZ'))

    x_siesta_input_ProjectionInSCF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ProjectionInSCF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ProjectionInSCF'))

    x_siesta_input_ProjectionMethod = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ProjectionMethod"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ProjectionMethod'))

    x_siesta_input_RcSpatial = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "RcSpatial"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_RcSpatial'))

    x_siesta_input_ReInitialiseDM = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ReInitialiseDM"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ReInitialiseDM'))

    x_siesta_input_ReadHamiltonian = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ReadHamiltonian"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ReadHamiltonian'))

    x_siesta_input_ReadKPIN = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ReadKPIN"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ReadKPIN'))

    x_siesta_input_ReparametrizePseudos = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ReparametrizePseudos"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ReparametrizePseudos'))

    x_siesta_input_Restricted_Radial_Grid = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Restricted.Radial.Grid"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Restricted.Radial.Grid'))

    x_siesta_input_Rmax_Radial_Grid = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Rmax.Radial.Grid"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Rmax.Radial.Grid'))

    x_siesta_input_RotateSpin_Phi = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "RotateSpin.Phi"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_RotateSpin.Phi'))

    x_siesta_input_RotateSpin_Theta = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "RotateSpin.Theta"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_RotateSpin.Theta'))

    x_siesta_input_SCF_LinearMixingAfterPulay = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.LinearMixingAfterPulay"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.LinearMixingAfterPulay'))

    x_siesta_input_SCF_MixAfterConvergence = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.MixAfterConvergence"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.MixAfterConvergence'))

    x_siesta_input_SCF_MixingWeightAfterPulay = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.MixingWeightAfterPulay"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.MixingWeightAfterPulay'))

    x_siesta_input_SCF_Pulay_Damping = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.Pulay.Damping"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.Pulay.Damping'))

    x_siesta_input_SCF_Pulay_DebugSVD = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.Pulay.DebugSVD"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.Pulay.DebugSVD'))

    x_siesta_input_SCF_Pulay_RcondSVD = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.Pulay.RcondSVD"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.Pulay.RcondSVD'))

    x_siesta_input_SCF_Pulay_UseSVD = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.Pulay.UseSVD"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.Pulay.UseSVD'))

    x_siesta_input_SCF_PulayDmaxRegion = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.PulayDmaxRegion"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.PulayDmaxRegion'))

    x_siesta_input_SCF_PulayMinimumHistory = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.PulayMinimumHistory"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.PulayMinimumHistory'))

    x_siesta_input_SCF_Read_Charge_NetCDF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.Read.Charge.NetCDF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.Read.Charge.NetCDF'))

    x_siesta_input_SCF_Read_Deformation_Charge_NetCDF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.Read.Deformation.Charge.NetCDF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.Read.Deformation.Charge.NetCDF'))

    x_siesta_input_SCF_Recompute_H_After_Scf = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.Recompute-H-After-Scf"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.Recompute-H-After-Scf'))

    x_siesta_input_SCF_Want_Variational_EKS = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCF.Want.Variational.EKS"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCF.Want.Variational.EKS'))

    x_siesta_input_SCFMustConverge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SCFMustConverge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SCFMustConverge'))

    x_siesta_input_SIC_Flavour = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.Flavour"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.Flavour'))

    x_siesta_input_SIC_Lambda = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.Lambda"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.Lambda'))

    x_siesta_input_SIC_NoRelaxation = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.NoRelaxation"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.NoRelaxation'))

    x_siesta_input_SIC_Npop = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.Npop"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.Npop'))

    x_siesta_input_SIC_PopDMConv = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.PopDMConv"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.PopDMConv'))

    x_siesta_input_SIC_PopKgridFactor = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.PopKgridFactor"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.PopKgridFactor'))

    x_siesta_input_SIC_PopSmatSparsity = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.PopSmatSparsity"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.PopSmatSparsity'))

    x_siesta_input_SIC_ProjectionMode = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.ProjectionMode"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.ProjectionMode'))

    x_siesta_input_SIC_ProjectionType = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.ProjectionType"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.ProjectionType'))

    x_siesta_input_SIC_Rot_Inv = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SIC.Rot_Inv"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SIC.Rot_Inv'))

    x_siesta_input_STT_Calculation = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "STT.Calculation"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_STT.Calculation'))

    x_siesta_input_STT_LinearResponse = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "STT.LinearResponse"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_STT.LinearResponse'))

    x_siesta_input_SaveBaderCharge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveBaderCharge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveBaderCharge'))

    x_siesta_input_SaveDeltaRho = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveDeltaRho"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveDeltaRho'))

    x_siesta_input_SaveElectrostaticPotential = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveElectrostaticPotential"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveElectrostaticPotential'))

    x_siesta_input_SaveHS = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveHS"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveHS'))

    x_siesta_input_SaveInitialChargeDensity = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveInitialChargeDensity"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveInitialChargeDensity'))

    x_siesta_input_SaveIonicCharge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveIonicCharge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveIonicCharge'))

    x_siesta_input_SaveNeutralAtomPotential = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveNeutralAtomPotential"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveNeutralAtomPotential'))

    x_siesta_input_SaveRho = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveRho"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveRho'))

    x_siesta_input_SaveRhoXC = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveRhoXC"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveRhoXC'))

    x_siesta_input_SaveTotalCharge = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveTotalCharge"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveTotalCharge'))

    x_siesta_input_SaveTotalPotential = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SaveTotalPotential"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SaveTotalPotential'))

    x_siesta_input_Scissor_Operator = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Scissor.Operator"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Scissor.Operator'))

    x_siesta_input_SetBulkTransvCell = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SetBulkTransvCell"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SetBulkTransvCell'))

    x_siesta_input_Siesta2Wannier90_NumberOfBands = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Siesta2Wannier90.NumberOfBands"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Siesta2Wannier90.NumberOfBands'))

    x_siesta_input_Siesta2Wannier90_NumberOfBandsDown = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Siesta2Wannier90.NumberOfBandsDown"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Siesta2Wannier90.NumberOfBandsDown'))

    x_siesta_input_Siesta2Wannier90_NumberOfBandsUp = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Siesta2Wannier90.NumberOfBandsUp"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Siesta2Wannier90.NumberOfBandsUp'))

    x_siesta_input_Siesta2Wannier90_WriteAmn = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Siesta2Wannier90.WriteAmn"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Siesta2Wannier90.WriteAmn'))

    x_siesta_input_Siesta2Wannier90_WriteEig = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Siesta2Wannier90.WriteEig"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Siesta2Wannier90.WriteEig'))

    x_siesta_input_Siesta2Wannier90_WriteMmn = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Siesta2Wannier90.WriteMmn"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Siesta2Wannier90.WriteMmn'))

    x_siesta_input_Siesta2Wannier90_WriteUnk = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Siesta2Wannier90.WriteUnk"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Siesta2Wannier90.WriteUnk'))

    x_siesta_input_Sigma_DSigmaDE = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Sigma.DSigmaDE"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Sigma.DSigmaDE'))

    x_siesta_input_SignatureRecords = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SignatureRecords"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SignatureRecords'))

    x_siesta_input_SimulateDoping = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SimulateDoping"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SimulateDoping'))

    x_siesta_input_SingleExcitation = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SingleExcitation"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SingleExcitation'))

    x_siesta_input_SkipLastIter = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SkipLastIter"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SkipLastIter'))

    x_siesta_input_SlabDipoleCorrection = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SlabDipoleCorrection"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SlabDipoleCorrection'))

    x_siesta_input_SolutionMethod = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SolutionMethod"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SolutionMethod'))

    x_siesta_input_SpinConfLeads = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SpinConfLeads"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SpinConfLeads'))

    x_siesta_input_SpinOrbit = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SpinOrbit"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SpinOrbit'))

    x_siesta_input_SpinPolarized = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SpinPolarized"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SpinPolarized'))

    x_siesta_input_SystemLabel = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SystemLabel"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SystemLabel'))

    x_siesta_input_SystemName = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "SystemName"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_SystemName'))

    x_siesta_input_TS_MixH = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "TS.MixH"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_TS.MixH'))

    x_siesta_input_TimeReversal = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "TimeReversal"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_TimeReversal'))

    x_siesta_input_TimeReversalSymmetryForKpoints = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "TimeReversalSymmetryForKpoints"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_TimeReversalSymmetryForKpoints'))

    x_siesta_input_TrCoefficients = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "TrCoefficients"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_TrCoefficients'))

    x_siesta_input_TryMemoryIncrease = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "TryMemoryIncrease"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_TryMemoryIncrease'))

    x_siesta_input_UseDomainDecomposition = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "UseDomainDecomposition"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_UseDomainDecomposition'))

    x_siesta_input_UseNewDiagk = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "UseNewDiagk"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_UseNewDiagk'))

    x_siesta_input_UseSaveData = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "UseSaveData"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_UseSaveData'))

    x_siesta_input_UseSpatialDecomposition = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "UseSpatialDecomposition"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_UseSpatialDecomposition'))

    x_siesta_input_UseStructFile = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "UseStructFile"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_UseStructFile'))

    x_siesta_input_UseTreeTimer = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "UseTreeTimer"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_UseTreeTimer'))

    x_siesta_input_VFinal = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "VFinal"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_VFinal'))

    x_siesta_input_VGate = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "VGate"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_VGate'))

    x_siesta_input_VInitial = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "VInitial"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_VInitial'))

    x_siesta_input_Vna_Filter = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "Vna.Filter"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_Vna.Filter'))

    x_siesta_input_WarningMinimumAtomicDistance = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WarningMinimumAtomicDistance"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WarningMinimumAtomicDistance'))

    x_siesta_input_WriteBands = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteBands"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteBands'))

    x_siesta_input_WriteCoorCerius = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteCoorCerius"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteCoorCerius'))

    x_siesta_input_WriteCoorInitial = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteCoorInitial"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteCoorInitial'))

    x_siesta_input_WriteCoorStep = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteCoorStep"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteCoorStep'))

    x_siesta_input_WriteCoorXmol = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteCoorXmol"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteCoorXmol'))

    x_siesta_input_WriteDM = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteDM"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteDM'))

    x_siesta_input_WriteDM_History_NetCDF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteDM.History.NetCDF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteDM.History.NetCDF'))

    x_siesta_input_WriteDM_NetCDF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteDM.NetCDF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteDM.NetCDF'))

    x_siesta_input_WriteDMHS_History_NetCDF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteDMHS.History.NetCDF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteDMHS.History.NetCDF'))

    x_siesta_input_WriteDMHS_NetCDF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteDMHS.NetCDF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteDMHS.NetCDF'))

    x_siesta_input_WriteDMT = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteDMT"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteDMT'))

    x_siesta_input_WriteDenchar = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteDenchar"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteDenchar'))

    x_siesta_input_WriteDiagdS = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteDiagdS"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteDiagdS'))

    x_siesta_input_WriteEDM = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteEDM"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteEDM'))

    x_siesta_input_WriteEigenvalues = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteEigenvalues"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteEigenvalues'))

    x_siesta_input_WriteForces = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteForces"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteForces'))

    x_siesta_input_WriteHSDeriv = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteHSDeriv"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteHSDeriv'))

    x_siesta_input_WriteHamiltonPop = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteHamiltonPop"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteHamiltonPop'))

    x_siesta_input_WriteHirshfeldPop = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteHirshfeldPop"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteHirshfeldPop'))

    x_siesta_input_WriteIonPlotFiles = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteIonPlotFiles"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteIonPlotFiles'))

    x_siesta_input_WriteKbands = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteKbands"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteKbands'))

    x_siesta_input_WriteKpoints = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteKpoints"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteKpoints'))

    x_siesta_input_WriteMDXmol = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteMDXmol"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteMDXmol'))

    x_siesta_input_WriteMDhistory = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteMDhistory"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteMDhistory'))

    x_siesta_input_WriteMullikenPop = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteMullikenPop"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteMullikenPop'))

    x_siesta_input_WriteProjections = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteProjections"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteProjections'))

    x_siesta_input_WriteSpinMulliken = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteSpinMulliken"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteSpinMulliken'))

    x_siesta_input_WriteSpinSCF = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteSpinSCF"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteSpinSCF'))

    x_siesta_input_WriteVNA = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteVNA"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteVNA'))

    x_siesta_input_WriteVoronoiPop = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteVoronoiPop"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteVoronoiPop'))

    x_siesta_input_WriteWaveFunctions = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "WriteWaveFunctions"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_WriteWaveFunctions'))

    x_siesta_input_XML_AbortOnErrors = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "XML.AbortOnErrors"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_XML.AbortOnErrors'))

    x_siesta_input_XML_AbortOnWarnings = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "XML.AbortOnWarnings"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_XML.AbortOnWarnings'))

    x_siesta_input_XML_Write = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "XML.Write"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_XML.Write'))

    x_siesta_input_ZBroadeningG = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZBroadeningG"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZBroadeningG'))

    x_siesta_input_ZLeftVcte = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZLeftVcte"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZLeftVcte'))

    x_siesta_input_ZM_CalcAllForces = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZM.CalcAllForces"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZM.CalcAllForces'))

    x_siesta_input_ZM_ForceTolAngle = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZM.ForceTolAngle"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZM.ForceTolAngle'))

    x_siesta_input_ZM_ForceTolLength = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZM.ForceTolLength"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZM.ForceTolLength'))

    x_siesta_input_ZM_MaxDisplAngle = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZM.MaxDisplAngle"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZM.MaxDisplAngle'))

    x_siesta_input_ZM_MaxDisplLength = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZM.MaxDisplLength"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZM.MaxDisplLength'))

    x_siesta_input_ZM_UnitsAngle = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZM.UnitsAngle"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZM.UnitsAngle'))

    x_siesta_input_ZM_UnitsLength = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZM.UnitsLength"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZM.UnitsLength'))

    x_siesta_input_ZRightVcte = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZRightVcte"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZRightVcte'))

    x_siesta_input_ZVGateL = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZVGateL"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZVGateL'))

    x_siesta_input_ZVGateR = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZVGateR"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZVGateR'))

    x_siesta_input_ZeemanTermBx = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZeemanTermBx"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZeemanTermBx'))

    x_siesta_input_ZeemanTermBy = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZeemanTermBy"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZeemanTermBy'))

    x_siesta_input_ZeemanTermBz = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "ZeemanTermBz"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_ZeemanTermBz'))

    x_siesta_input_alloc_report_level = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "alloc_report_level"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_alloc_report_level'))

    x_siesta_input_alloc_report_threshold = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "alloc_report_threshold"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_alloc_report_threshold'))

    x_siesta_input_blocksize = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "blocksize"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_blocksize'))

    x_siesta_input_compat_pre_v4_dynamics = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "compat-pre-v4-dynamics"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_compat-pre-v4-dynamics'))

    x_siesta_input_fdf_debug = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "fdf-debug"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_fdf-debug'))

    x_siesta_input_kgrid_cutoff = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "kgrid_cutoff"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_kgrid_cutoff'))

    x_siesta_input_processorY = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "processorY"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_processorY'))

    x_siesta_input_timer_report_threshold = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "timer_report_threshold"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_timer_report_threshold'))

    x_siesta_input_user_basis = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "user-basis"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_user-basis'))

    x_siesta_input_user_basis_netcdf = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "user-basis-netcdf"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_user-basis-netcdf'))

    x_siesta_input_xc_authors = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "xc.authors"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_xc.authors'))

    x_siesta_input_xc_functional = Quantity(
        type=str,
        shape=[],
        description='''
        siesta input variable "xc.functional"
        ''',
        categories=[x_siesta_input],
        a_legacy=LegacyDefinition(name='x_siesta_input_xc.functional'))


class section_method(public.section_method):

    m_def = Section(validate=False, extends_base_section=True, a_legacy=LegacyDefinition(name='section_method'))

    x_siesta_section_input = SubSection(
        sub_section=SectionProxy('x_siesta_section_input'),
        repeats=True,
        a_legacy=LegacyDefinition(name='x_siesta_section_input'))


m_package.__init_metainfo__()



from .entries import BaseResourceMapper

__all__ = ("LinksMapper",)


class LinksMapper(BaseResourceMapper):

    ENDPOINT = "links"

    @classmethod
    def map_back(cls, doc: dict) -> dict:
        """Map properties from MongoDB to OPTIMADE

        :param doc: A resource object in MongoDB format
        :type doc: dict

        :return: A resource object in OPTIMADE format
        :rtype: dict
        """
        type_ = doc["type"]
        newdoc = super().map_back(doc)
        newdoc["type"] = type_
        return newdoc



# Copyright 2015-2018 Mikkel Strange, Fawzi Mohamed, Ankit Kariryaa, Ask Hjorth Larsen, Jens Jørgen Mortensen
# 
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from __future__ import print_function
from builtins import str
from builtins import object
import tarfile
import xml.etree.ElementTree as ElementTree

import numpy as np


class Reader(object):
    def __init__(self, name):
        self.dims = {}
        self.shapes = {}
        self.dtypes = {}
        self.parameters = {}
        self.tar = tarfile.open(name, 'r')
        f = self.tar.extractfile('info.xml').read()
        root = ElementTree.fromstring(f)
        self.byteswap = ((root.attrib['endianness'] == 'little') !=
                         np.little_endian)
        for child in root:
            a = child.attrib
            if child.tag == 'parameter':
                try:
                    value = eval(a['value'], {})
                except (SyntaxError, NameError):
                    value = str(a['value'])
                self.parameters[a['name']] = value
            elif child.tag == 'array':
                name = a['name']
                self.dtypes[name] = a['type']
                self.shapes[name] = []
                for dim in child:
                    n = int(dim.attrib['length'])
                    self.shapes[name].append(n)
                    self.dims[dim.attrib['name']] = n

    def __contains__(self, name):
        return name in self.parameters or name in self.shapes

    def __getattr__(self, name):
        if name in self.parameters:
            return self.parameters[name]
        else:
            return self.get_array(name)

    def get_array(self, name):
        fileobj = self.tar.extractfile(name)
        shape = self.shapes[name]
        dtype = self.dtypes[name]
        dtype = np.dtype({'int': 'int32'}.get(dtype, dtype))
        size = np.prod(shape) * dtype.itemsize
        array = np.fromstring(fileobj.read(size), dtype)
        if self.byteswap:
            array = array.byteswap()
        if dtype == np.int32:
            array = np.asarray(array, int)
        array.shape = shape
        return array

    def close(self):
        self.tar.close()


if __name__ == '__main__':
    r = Reader('H2.gpw')
    print(r.Eigenvalues, r.FermiLevel, r.UnitCell)
    print('UnitCell' in r, 'sdgsdgsdfg' in r)



# Copyright 2016-2018 Arvid Conrad Ihrig, Aliaksei Mazheika
#                     Fritz-Haber-Institut der Max-Planck-Gesellschaft
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module constructs the parser for the GRAD module from TurboMole"""

import logging
import re
from nomadcore.simple_parser import SimpleMatcher as SM
from turbomoleparser.TurbomoleCommon import RE_FLOAT
import turbomoleparser.TurbomoleCommon as Common

logger = logging.getLogger("nomad.turbomoleParser")


class GRADparser(object):

    def __init__(self, context, key="grad"):
        context[key] = self
        self.__context = context
        self.__backend = None

    def purge_data(self):
        pass

    def set_backend(self, backend):
        self.__backend = backend

    def build_parser(self):
        references = SM(r"\s{5,}[^+ ]+",
                        name="references",
                        coverageIgnore=True,
                        repeats=True,
                        )
        header = SM(r"\s*g r a d - program\s*$",
                    name="header",
                    coverageIgnore=True,
                    subMatchers=[references],
                    endReStr=r"\s*\+-+\+"
                    )

        sub_matchers = [
            header,
            self.__context["geo"].build_qm_geometry_matcher(),
            self.__context["geo"].build_orbital_basis_matcher(),
            self.__context["orbitals"].build_ir_rep_matcher(),
            self.__context["method"].build_dft_functional_matcher(),
            self.__context["method"].build_dftd3_vdw_matcher(),
            self.__context["gradient"].build_gradient_matcher(),
            Common.build_profiling_matcher(r"\s*grad(?:\.all)? profiling\s*$"),
        ]

        return self.__context.build_module_matcher("grad", sub_matchers, "GRAD",
                                                   self.__context["method"].add_default_functional)



# Copyright 2019 Alvin Noe Ladines, Markus Scheidgen
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Generates a msgpack database of springer-related quantities downloaded from
http://materials.springer.com. The database is stuctured as

space_group_number : normalized_formula : springer_id : entry
'''

from typing import Dict, List, Any
import requests
import re
import bs4
import time
import os.path

from nomad import config, archive


required_items = {
    'Alphabetic Formula:': 'alphabetic_formula',
    'Classification by Properties:': 'classification',
    'Compound Class(es):': 'compound_classes',
    'Dataset ID': 'id',
    'Space Group:': 'space_group_number',
    'Phase Label(s):': 'phase_labels'
}

spaces_re = re.compile(r'\s+')
search_re = re.compile(" href=\"(/isp/[^\"]+)")
formula_re = re.compile(r'([A-Z][a-z]?)([0-9.]*)|\[(.*?)\]([0-9]+)')


def _update_dict(dict0: Dict[str, float], dict1: Dict[str, float]):
    for key, val in dict1.items():
        if key in dict0:
            dict0[key] += val
        else:
            dict0[key] = val


def _components(formula_str: str, multiplier: float = 1.0) -> Dict[str, float]:
    # match atoms and molecules (in brackets)
    components = formula_re.findall(formula_str)

    symbol_amount: Dict[str, float] = {}
    for component in components:
        element, amount_e, molecule, amount_m = component
        if element:
            if not amount_e:
                amount_e = 1.0
            _update_dict(symbol_amount, {element: float(amount_e) * multiplier})

        elif molecule:
            if not amount_m:
                amount_m = 1.0
            _update_dict(symbol_amount, _components(molecule, float(amount_m) * multiplier))

    return symbol_amount


def normalize_formula(formula_str: str) -> str:
    symbol_amount = _components(formula_str)

    total = sum(symbol_amount.values())
    symbol_normamount = {e: round(a / total * 100.) for e, a in symbol_amount.items()}

    formula_sorted = [
        '%s%d' % (s, symbol_normamount[s]) for s in sorted(list(symbol_normamount.keys()))]

    return ''.join(formula_sorted)


def parse(htmltext: str) -> Dict[str, str]:
    '''
    Parser the quantities in required_items from an html text.
    '''
    soup = bs4.BeautifulSoup(htmltext, "html.parser")
    results = {}
    for item in soup.find_all(attrs={"class": "data-list__item"}):
        key = item.find(attrs={"class": "data-list__item-key"})
        if not key:
            continue

        key_str = key.string.strip()
        if key_str not in required_items:
            continue

        value = item.find(attrs={"class": "data-list__item-value"})
        value = spaces_re.sub(' ', value.get_text()).strip()
        results[required_items[key_str]] = value

        if len(results) >= len(required_items):
            break

    if 'classification' in results:
        results['classification'] = [x.strip() for x in results['classification'].split(",")]
        results['classification'] = [x for x in results['classification'] if x != '–']
    if 'compound_classes' in results:
        results['compound_classes'] = [x.strip() for x in results['compound_classes'].split(",")]
        results['compound_classes'] = [x for x in results['compound_classes'] if x != '–']

    normalized_formula = None
    for formula_type in ['alphabetic_formula', 'phase_labels']:
        formula = results.get(formula_type, None)
        if formula:
            try:
                normalized_formula = normalize_formula(formula)
                break
            except Exception:
                pass

    results['normalized_formula'] = normalized_formula

    return results


def _merge_dict(dict0: Dict[str, Any], dict1: Dict[str, Any]) -> Dict[str, Any]:
    if not isinstance(dict1, dict) or not isinstance(dict0, dict):
        return dict1

    for k, v in dict1.items():
        if k in dict0:
            dict0[k] = _merge_dict(dict0[k], v)
        else:
            dict0[k] = v
    return dict0


def _download(path: str, max_n_query: int = 10, retry_time: int = 120) -> str:
    n_query = 0
    while True:
        response = requests.get(path)
        if response.status_code == 200:
            break
        if n_query > max_n_query:
            break
        n_query += 1
        time.sleep(retry_time)

    if response.status_code != 200:
        response.raise_for_status()

    return response.text


def update_springer(max_n_query: int = 10, retry_time: int = 120):
    '''
    Downloads the springer quantities related to a structure from springer and updates
    database.
    '''
    # load database
    # querying database with unvailable dataset leads to error,
    # get toc keys first by making an empty query
    springer_archive = archive.ArchiveReader(config.normalize.springer_db_path)
    _ = springer_archive._load_toc_block(0)
    archive_keys = springer_archive._toc.keys()

    sp_data = archive.query_archive(config.normalize.springer_db_path, {spg: '*' for spg in archive_keys})

    sp_ids: List[str] = []
    for spg in sp_data:
        if not isinstance(sp_data[spg], dict):
            continue
        for formula in sp_data[spg]:
            sp_ids += list(sp_data[spg][formula].keys())

    page = 1
    while True:
        # check springer database for new entries by comparing with local database
        root = 'http://materials.springer.com/search?searchTerm=&pageNumber=%d&datasourceFacet=sm_isp&substanceId=' % page
        req_text = _download(root, max_n_query, retry_time)
        if 'Sorry,' in req_text:
            break

        paths = search_re.findall(req_text)

        if len(paths) == 0:
            break

        for path in paths:
            sp_id = os.path.basename(path)
            if sp_id in sp_ids:
                continue

            path = 'http://materials.springer.com%s' % path
            req_text = _download(path, max_n_query, retry_time)
            try:
                data = parse(req_text)
            except Exception:
                continue

            space_group_number = data.get('space_group_number', None)
            normalized_formula = data.get('normalized_formula', None)
            if space_group_number is None or normalized_formula is None:
                continue
            aformula = data.get('alphabetic_formula', None)
            if aformula is None:
                aformula = data.get('phase_labels', None)
            compound = data.get('compound_classes', None)
            classification = data.get('classification', None)

            entry = dict(
                aformula=aformula, url=path, compound=compound,
                classification=classification)
            sp_data = _merge_dict(
                sp_data, {str(space_group_number): {normalized_formula: {sp_id: entry}}})

        page += 1

    archive.write_archive(
        config.normalize.springer_db_path, len(sp_data), sp_data.items(),
        entry_toc_depth=1)



from typing import Union

from fastapi import APIRouter, Depends, Request

from optimade.models import (
    ErrorResponse,
    ReferenceResource,
    ReferenceResponseMany,
    ReferenceResponseOne,
)
from optimade.server.config import CONFIG
from optimade.server.entry_collections import MongoCollection, client
from optimade.server.mappers import ReferenceMapper
from optimade.server.query_params import EntryListingQueryParams, SingleEntryQueryParams

from .utils import get_entries, get_single_entry


router = APIRouter(redirect_slashes=True)

references_coll = MongoCollection(
    collection=client[CONFIG.mongo_database][CONFIG.references_collection],
    resource_cls=ReferenceResource,
    resource_mapper=ReferenceMapper,
)


@router.get(
    "/references",
    response_model=Union[ReferenceResponseMany, ErrorResponse],
    response_model_exclude_unset=True,
    tags=["References"],
)
def get_references(request: Request, params: EntryListingQueryParams = Depends()):
    return get_entries(
        collection=references_coll,
        response=ReferenceResponseMany,
        request=request,
        params=params,
    )


@router.get(
    "/references/{entry_id:path}",
    response_model=Union[ReferenceResponseOne, ErrorResponse],
    response_model_exclude_unset=True,
    tags=["References"],
)
def get_single_reference(
    request: Request, entry_id: str, params: SingleEntryQueryParams = Depends()
):
    return get_single_entry(
        collection=references_coll,
        entry_id=entry_id,
        response=ReferenceResponseOne,
        request=request,
        params=params,
    )



# Copyright 2015-2018 Ask Hjorth Larsen, Fawzi Mohamed, Ankit Kariryaa
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

#
# Main author and maintainer: Ask Hjorth Larsen <asklarsen@gmail.com>


from __future__ import print_function
import re
import os
import sys
from contextlib import contextmanager

import numpy as np
import logging
from ase.units import Bohr
from ase.io import read

# import setup_paths

from nomadcore.unit_conversion.unit_conversion import convert_unit

from octopusparser.aseoct import Octopus, parse_input_file

"""This is the Octopus parser.

It has to parse many files:
 * input file, 'inp' (ASE does this)
 * output file, 'static/info' (SimpleParser)
   - similar file or files for other calculation modes
 * anything written to stdout (this file could have any name) (SimpleParser)
   - program output can be cooking recipes ("parse *all* outputs")
 * geometry input file, if specified in 'inp'
   - ASE takes care of that
 * output density/potential/etc. if written to files
   - cube
   - xcrysden
   - xyz
   - other candidates: vtk, etsf

There are more output formats but they are non-standard, or for
debugging, not commonly used.  We will only implement support for
those if many uploaded calculations contain those formats.  I think it
is largely irrelevant.
"""

metaInfoEnv = None


def parse_infofile(meta_info_env, pew, fname):
    # print('\n\n### parse_infofile()')
    # print('\tPROBLEM: {}\n\t{} ' .format(fname, 'should be static/info!!'))
    with open(fname) as fd:
        for line in fd:
            # print(line)
            if line.startswith('SCF converged'):
                iterations = int(line.split()[-2])
                pew.addValue('x_octopus_info_scf_converged_iterations',
                             iterations)

        for line in fd:  # Jump down to energies:
            if line.startswith('Energy ['):
                octunit = line.strip().split()[-1].strip('[]:')
                nomadunit = {'eV': 'eV', 'H': 'hartree'}[octunit]
                break

        names = {'Total': 'energy_total',
                 'Free': 'energy_free',
                 'Ion-ion': 'x_octopus_info_energy_ion_ion',
                 'Eigenvalues': 'energy_sum_eigenvalues',
                 'Hartree': 'energy_electrostatic',
                 'Exchange': 'energy_X',
                 'Correlation': 'energy_C',
                 'vanderWaals': 'energy_van_der_Waals',
                 '-TS': 'energy_correction_entropy',
                 'Kinetic': 'electronic_kinetic_energy'}

        for line in fd:
            if line.startswith('---'):
                continue
            tokens = line.split()
            if len(tokens) < 3:
                break

            if tokens[0] in names:
                pew.addValue(names[tokens[0]],
                             convert_unit(float(tokens[2]), nomadunit))


def parse_logfile(meta_info_env, pew, fname):
    '''Parse the mainfile'''
    maxlines = 100
    with open(fname) as fd:
        for i, line in enumerate(fd):
            if i > maxlines:
                break
            if line.startswith('Version'):
                version = line.split()[-1]
                pew.addValue('program_version', version)
            elif line.startswith('Revision'):
                revision = int(line.split()[-1])
                pew.addValue('x_octopus_log_svn_revision', revision)


def parse_gridinfo(metaInfoEnv, pew, fname):
    '''
    fname: main filename
    '''
    results = {}

    with open(fname) as fd:
        for line in fd:
            if '*** Grid ***' in line:
                break

        line = next(fd)

        while '***********************' not in line:
            if line.startswith('Simulation Box:'):
                line = next(fd)

                while line == '' or line[0].isspace():
                    m = re.match(r'\s*Type\s*=\s*(\S+)', line)
                    if m:
                        results['boxshape'] = m.group(1)
                    m = re.match(r'\s*Octopus will treat the system as periodic in (\S+) dim', line)
                    if m:
                        results['npbc'] = int(m.group(1))

                    m = re.match(r'\s*Lattice Vectors \[(\S+)\]', line)
                    if m:
                        results['unit'] = m.group(1)
                        cell = np.array([[float(x) for x in next(fd).split()] for _ in range(3)])
                        results['cell'] = cell

                    line = next(fd)

            if line.startswith('Main mesh:'):
                line = next(fd)
                while line == '' or line[0].isspace():
                    m = re.match(r'\s*Spacing\s*\[(\S+)\]\s*=\s*\(\s*(\S+?),\s*(\S+?),\s*(\S+?)\)', line)
                    if m:
                        results['unit'] = m.group(1)
                        results['spacing'] = np.array(m.group(2, 3, 4)).astype(float)

                    line = next(fd)
    return results


def parse_coordinates_from_parserlog(fname):
    results = {}

    def buildblock(block):
        imax = 1 + max(b[0] for b in block)
        jmax = 1 + max(b[1] for b in block)
        array = np.zeros((imax, jmax), object)
        for i, j, val in block:
            array[i, j] = val
        labels = np.array([val[1:-1] for val in array[:, 0].astype(str)])
        numbers = array[:, 1:4].astype(float)
        return labels, numbers

    with open(fname) as fd:
        blockname = None
        block = None
        for line in fd:
            line.split('#')[0].strip()

            m = re.match(r"Opened block '(Coordinates|ReducedCoordinates)'", line)
            if m:
                blockname = m.group(1)
                block = []
                continue

            m = re.match(r'Closed block', line)
            if m:
                if blockname is None:
                    continue  # This is some other block
                labels, numbers = buildblock(block)
                results['labels'] = labels
                if blockname == 'Coordinates':
                    results['coords'] = numbers
                else:
                    assert blockname == 'ReducedCoordinates'
                    results['rcoords'] = numbers

                blockname = None
                block = None
                continue

            if block is not None:
                # Example:   (0, 0) = "Ag"
                paren_expression = r'\s*\((\d+),\s*(\d+)\)\s*=\s*(.+)'
                m = re.match(paren_expression, line)
                if not m:
                    # Example:   Coordinates (0, 0) = "O"
                    m = re.match(r'\s*' + blockname + paren_expression, line)
                if not m:
                    # Example:  Coordinates[0][0] = "H"
                    m = re.match(r'\s*' + blockname + r'\s*\[(\d+)\]\[(\d+)\]\s*=\s*(.+)', line)

                assert m is not None
                i, j, val = m.group(1, 2, 3)
                block.append((int(i), int(j), val))

            for name in ['PDBCoordinates',
                         'XYZCoordinates',
                         'XSFCoordinates']:
                m = re.match(name + r'\s*=\s*"(.+?)"', line)
                if m:
                    assert block is None
                    results[name] = m.group(1)

            m = re.match('XSFCoordinatesAnimStep\s*=\s*(\d+)', line)
            if m:
                results['XSFCoordinatesAnimStep'] = int(m.group(1))

            for name in ['Units',
                         'UnitsInput']:
                m = re.match(r'\s*' + name + r'\s*=\s*(\S+)', line)
                if m:
                    results[name] = m.group(1)

    return results


def normalize_names(names):
    return [name.lower() for name in names]


# Dictionary of all meta info:
normalized2real = None


parser_info = {
    "name": "parser_octopus",
    "version": "1.0"
}


def read_parser_log(path):
    exec_kwargs = {}
    # print("\n\nPATH: ", path) # 'exec/parser.log'
    with open(path) as fd:
        for line in fd:
            # Remove comment:
            line = line.split('#', 1)[0].strip()
            tokens = line.split('=')
            try:
                name, value = tokens
            except ValueError:
                continue  # Not an assignment
            name = name.strip().lower()
            value = value.strip()

            if ' ' in name:
                # Not a real name
                continue
            exec_kwargs[name] = value
    return exec_kwargs


def is_octopus_logfile(fname):
    try:
        with open(fname) as fd:
            for n, line in enumerate(fd):
                if n > 20:
                    break
                if '|0) ~ (0) |' in line:  # Eyes from Octopus logo
                    return True
    except UnicodeDecodeError:
        # ingnore files that cannot be decoded, there are not the files
        # we are looking for
        pass

    return False


def find_octopus_logfile(dirname):
    for fname in os.listdir(dirname):
        fname = os.path.join(dirname, fname)
        if os.path.isfile(fname) and is_octopus_logfile(fname):
            return fname
    return None


def override_keywords(kwargs, parser_log_kwargs):
    # Some variables we can get from the input file, but they may
    # contain arithmetic and variable assignments which cannot
    # just be parsed into a final value.  The output of the
    # Octopus parser (exec/parser.log) is reduced to pure numbers,
    # so that is useful, except when the variable was a name (such
    # as an eigensolver).  In that case we should definitely not
    # rely on the number!  We will take some variables from the
    # exec/parser.log but most will just be verbatim from the
    # input file whether they can be parsed or not.
    exec_override_keywords = set(['radius',
                                  # 'lsize',
                                  'spacing'])

    outkwargs = kwargs.copy()

    # Now override any relevant keywords:
    for name in exec_override_keywords:
        if name in kwargs and name in parser_log_kwargs:
            if name == 'lsize':
                lsize = []
                for i in range(3):
                    # (This is relatively horrible.  We are looking for
                    # lines like "Lsize (0, 1) = 5.0" or similar)
                    lsize_tmp = 'lsize (0, %d)' % i
                    assert lsize_tmp in parser_log_kwargs
                    lsize.append(parser_log_kwargs[lsize_tmp])
                outkwargs[name] = [lsize]
                continue

            # print('Keyword %s with value %s overridden by value '
            #       '%s obtained from parser log'
            #       % (name, kwargs[name], parser_log_kwargs[name]),
            #       file=fd)
            logging.debug(
                ('Keyword %s with value %s overridden by value '
                 '%s obtained from parser log')
                % (name, kwargs[name], parser_log_kwargs[name]))

            outkwargs[name] = parser_log_kwargs[name]
    return outkwargs


metadata_dtypes = {'b': bool,
                   'C': str,
                   'f': float}  # Integer?


# Convert (<normalized name>, <extracted string>) into
# (<real metadata name>, <value of correct type>)
def regularize_metadata_entry(normalized_name, value):
    realname = normalized2real[normalized_name]
    assert realname in metaInfoEnv, 'No such metadata: %s' % realname
    metainfo = metaInfoEnv[realname]
    dtype = metainfo['dtypeStr']
    converted_value = metadata_dtypes[dtype](value)
    return realname, converted_value


def register_octopus_keywords(pew, category, kwargs):
    skip = set(['mixingpreconditioner', 'mixinterval'])
    for keyword in kwargs:
        if keyword in skip:  # XXXX
            continue
        # How do we get the metadata type?
        normalized_name = 'x_octopus_%s_%s' % (category, keyword)
        val = kwargs[keyword]
        try:
            name, value = regularize_metadata_entry(normalized_name, val)
        except Exception:  # unknown normalized_name or cannot convert
            pass
            # We can't crash on unknown keywords because we must support
            # versions old and new alike.
            #
            # Some keywords (e.g. Spacing) specify float as type, but they
            # can actually be blocks.  (block is a type itself)
        else:
            pew.addValue(name, value)


def parse_without_class(fname, backend, parser_info):
    # print('# ', fname )
    # fname refers to the static/info file.
    # Look for files before we create some of our own files for logging etc.:
    # fd = stdout # Print output to stdout
    absfname = os.path.abspath(fname)
    # staticdirname, _basefname = os.path.split(absfname)
    # dirname, _static = os.path.split(staticdirname)
    dirname = os.path.dirname(os.path.abspath(fname))
    # assert _static == 'static'
    inp_path = os.path.join(dirname, 'inp')
    parser_log_path = os.path.join(dirname, 'exec', 'parser.log')
    logfile = find_octopus_logfile(dirname)

    # pew.startedParsingSession(fname, parser_info)

    pew = backend
    pew.startedParsingSession(fname, parser_info)

    # this context manager shamelessly copied from GPAW parser
    # Where should Python code be put if it is used by multiple parsers?
    @contextmanager
    def open_section(name):
        gid = pew.openSection(name)
        yield gid
        pew.closeSection(name, gid)

    with open_section('section_run'):
        pew.addValue('program_name', 'Octopus')
        pew.addValue('program_basis_set_type', 'Real-space grid')

        # print(file=fd)
        # print('Read Octopus keywords from input file %s' % inp_path,
        #       file=fd)
        logging.debug('Read Octopus keywords from input file %s' % inp_path)

        with open(inp_path) as inp_fd:
            kwargs = parse_input_file(inp_fd)
        register_octopus_keywords(pew, 'input', kwargs)

        # print('Read processed Octopus keywords from octparse logfile %s'
        #       % parser_log_path, file=fd)
        logging.debug(
            'Read processed Octopus keywords from octparse logfile %s' % parser_log_path)

        parser_log_kwargs = read_parser_log(parser_log_path)
        register_octopus_keywords(pew, 'parserlog', parser_log_kwargs)

        # print('Override certain keywords with processed keywords', file=fd)
        kwargs = override_keywords(kwargs, parser_log_kwargs)

        # print('Read as ASE calculator', file=fd)
        calc = Octopus(dirname)

        with open_section('section_basis_set_cell_dependent') as basis_set_cell_dependent_gid:
            pew.addValue('basis_set_cell_dependent_kind', 'realspace_grids')
            # How does nomad want grid spacing?

        nbands = calc.get_number_of_bands()
        nspins = calc.get_number_of_spins()
        nkpts = len(calc.get_k_point_weights())

        # fermi_energy = calc.get_fermi_level()
        # print('I can see: ', fermi_energy)
        # pew.addArrayValues('energy_reference_fermi', [fermi_energy, fermi_energy])

        if logfile is None:
            # print('No stdout logfile found', file=fd)
            logging.debug('No stdout logfile found')
        else:
            # print('Found stdout logfile %s' % logfile, file=fd)
            logging.debug('Found stdout logfile %s' % logfile)
            # print('Parse logfile %s' % logfile, file=fd)
            logging.debug('Parse logfile %s' % logfile)
            parse_logfile(metaInfoEnv, pew, logfile)

        # print('Add parsed values', file=fd)
        with open_section('section_system') as system_gid:
            gridinfo = parse_gridinfo(metaInfoEnv, pew, fname)
            cell_unit = gridinfo['unit']
            if 'cell' in gridinfo:
                cell = gridinfo['cell']
                if cell_unit == 'A':
                    cell /= Bohr  # cell guaranteed to be Bohr now
                else:
                    assert cell_unit == 'b'
                pew.addArrayValues('simulation_cell',
                                   convert_unit(cell, 'bohr'))

            # We will get the positions from the parser log.
            # It is the only "practical" way.
            coordinfo = parse_coordinates_from_parserlog(parser_log_path)
            atoms = None
            coords = None

            # Old versions allow this abomination.
            if kwargs.get('unitsinput', kwargs.get('units')) == 'ev_angstrom':
                input_units = 'angstrom'
            else:
                input_units = 'bohr'

            parent_dir, _ = os.path.split(fname)
            inpdir = parent_dir
            # try to read files from the same or parent directory
            while True:
                try:
                    if 'PDBCoordinates' in coordinfo:
                        atoms = read(os.path.join(inpdir, coordinfo['PDBCoordinates']), format='proteindatabank')
                    elif 'XYZCoordinates' in coordinfo:
                        atoms = read(os.path.join(inpdir, coordinfo['XYZCoordinates']), format='xyz')
                    elif 'XSFCoordinates' in coordinfo:
                        if 'XSFCoordinatesAnimStep' in coordinfo:
                            assert 0  # XXX read correct step.  Take 1-indexation into account
                        atoms = read(os.path.join(inpdir, coordinfo['XSFCoordinates']), format='xsf')
                    elif 'coords' in coordinfo:
                        coords = coordinfo['coords']
                        if input_units == 'angstrom':
                            coords /= Bohr
                    elif 'rcoords' in coordinfo:
                        # unit will be Bohr cf. handling of cell above
                        coords = np.dot(coordinfo['rcoords'], cell)
                except FileNotFoundError as e:
                    if inpdir == parent_dir:
                        inpdir, _ = os.path.split(inpdir)
                    else:
                        raise e
                else:
                    break

            if atoms is not None:
                coords = atoms.positions / Bohr

            assert coords is not None, 'Cannot find coordinates'
            labels = coordinfo.get('labels')
            if labels is None:
                labels = np.array(atoms.get_chemical_symbols())

            pbc = np.zeros(3)

            if 'npbc' in gridinfo:
                pbc = np.zeros(3, bool)
                pbc[:gridinfo['npbc']] = True
                pew.addArrayValues('configuration_periodic_dimensions', pbc)

            pew.addArrayValues('atom_labels', labels)
            pew.addArrayValues('atom_positions', convert_unit(coords, 'bohr'))

            # XXX FIXME atoms can be labeled in ways not compatible with ASE.
            # pew.addArrayValues('atom_labels',
            #                   np.array(atoms.get_chemical_symbols()))
            # pew.addArrayValues('atom_positions',
            #                   convert_unit(atoms.get_positions(), 'angstrom'))
            # pew.addArrayValues('configuration_periodic_dimensions',
            #                   np.array(atoms.pbc))

        with open_section('section_single_configuration_calculation'):
            fermi_energy = calc.get_fermi_level()
            pew.addArrayValues('energy_reference_fermi', [fermi_energy, fermi_energy])

            pew.addValue('single_configuration_calculation_to_system_ref',
                         system_gid)
            # print('Parse info file %s' % fname) #, file=fd)
            logging.debug('Parse info file %s' % fname)  # mainfile
            parse_infofile(metaInfoEnv, pew, fname)

            with open_section('section_method') as method_gid:
                for basis_set_kind in ['density', 'wavefunction']:
                    with open_section('section_method_basis_set'):
                        pew.addValue('method_basis_set_kind', basis_set_kind)
                        pew.addValue('mapping_section_method_basis_set_cell_associated',
                                     basis_set_cell_dependent_gid)
                # smearing_width = float(kwargs.get('smearing', 0.0))
                # pew.addValue('smearing_width',
                #             convert_unit(smearing_width, ENERGY_UNIT))
                # XXX remember to get smearing width somehow
                smearing_func = kwargs.get('smearingfunction',
                                           'semiconducting')
                smearing_kinds = {'semiconducting': 'empty',
                                  'spline_smearing': 'gaussian',
                                  # Note: spline and Gaussian are only
                                  # nearly identical.  See:
                                  # oct-help --print SmearingFunction
                                  'fermi_dirac': 'fermi',
                                  'cold_smearing': 'marzari-vanderbilt',
                                  'methfessel_paxton': 'methfessel-paxton'}
                                  # '': 'tetrahedra',

                pew.addValue('smearing_kind',
                             smearing_kinds[smearing_func])

                pew.addValue('number_of_spin_channels', nspins)
                pew.addValue('total_charge',
                             float(parser_log_kwargs['excesscharge']))
                oct_theory_level = kwargs.get('theorylevel', 'dft')

                theory_levels = dict(independent_particles='independent_particles',  # not defined by Nomad
                                     hartree='Hartree',  # not defined by Nomad
                                     hartree_fock='DFT',  # nomad wants it that way
                                     dft='DFT',
                                     classical='Classical',  # Not defined by Nomad
                                     rdmft='RDMFT')  # Not defined by Nomad
                # TODO how do we warn if we get an unexpected theory level?
                # Currently: KeyError
                nomad_theory_level = theory_levels[oct_theory_level]

                pew.addValue('electronic_structure_method', nomad_theory_level)

                if oct_theory_level == 'dft':
                    ndim = int(kwargs.get('dimensions', 3))
                    assert ndim in range(1, 4), ndim
                    default_xc = ['lda_x_1d + lda_c_1d_csc',
                                  'lda_x_2d + lda_c_2d_amgb',
                                  'lda_x + lda_c_pz_mod'][ndim - 1]
                    xcfunctional = kwargs.get('xcfunctional', default_xc)
                    for functional in xcfunctional.split('+'):
                        functional = functional.strip().upper()
                        with open_section('section_XC_functionals'):
                            pew.addValue('XC_functional_name', functional)

                forces = calc.results.get('forces')
                if forces is not None:
                    pew.addArrayValues('atom_forces_free_raw',
                                       convert_unit(forces, 'eV'))
                # Convergence parameters?

            pew.addValue('single_configuration_to_calculation_method_ref',
                         method_gid)

            with open_section('section_eigenvalues'):
                if kwargs.get('theorylevel', 'dft') == 'dft':
                    pew.addValue('eigenvalues_kind', 'normal')

                kpts = calc.get_ibz_k_points()
                assert len(kpts) == nkpts
                pew.addValue('number_of_eigenvalues_kpoints', nkpts)

                eig = np.zeros((nspins, nkpts, nbands))
                occ = np.zeros((nspins, nkpts, nbands))

                for s in range(nspins):
                    for k in range(nkpts):
                        eig[s, k, :] = calc.get_eigenvalues(kpt=k, spin=s)
                        occ[s, k, :] = calc.get_occupation_numbers(kpt=k,
                                                                   spin=s)
                pew.addArrayValues('eigenvalues_kpoints', kpts)
                pew.addArrayValues('eigenvalues_values',
                                   convert_unit(eig, 'eV'))
                pew.addArrayValues('eigenvalues_occupation', occ)

    pew.finishedParsingSession('ParseSuccess', None)
    return backend


class OctopusParserWrapper():
    """ A proper class envolop for running this parser using Noamd-FAIRD infra. """
    def __init__(self, backend, **kwargs):
        self.backend_factory = backend

    def parse(self, mainfile):
        logging.info('octopus parser started')
        logging.getLogger('nomadcore').setLevel(logging.WARNING)
        backend = self.backend_factory("octopus.nomadmetainfo.json")

        # We need access to this information because we want/need to dynamically convert
        # extracted metadata to its correct type.  Thus we need to know the type.
        # Also since input is case insensitive, we need to convert normalized (lowercase)
        # metadata names to their real names which are normally CamelCase.
        global metaInfoEnv
        if metaInfoEnv is None:
            metaInfoEnv = backend.metaInfoEnv()
            metaInfoKinds = metaInfoEnv.infoKinds.copy()
            all_metadata_names = list(metaInfoKinds.keys())
            global normalized2real
            normalized2real = dict(zip(normalize_names(all_metadata_names), all_metadata_names))

        # Call the old parser without a class.
        parserInfo = parser_info
        backend = parse_without_class(mainfile, backend, parserInfo)
        return backend


if __name__ == '__main__':
    fname = sys.argv[1]
    logfname = 'parse.log'
    # print('pppp')
    with open(logfname, 'w') as fd:
        parse(fname, fd)
